package trains.model;

import java.time.DayOfWeek;
import java.time.Duration;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class Train {
	private static final DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("E", Locale.ITALY);
	
	private Operator operator;
	private long number;
	private DayOfWeek[] days;
	private List<Stop> stops;

	public Train(Operator operator, long number, DayOfWeek[] days,
			List<Stop> stops) {
		if (operator == null)
			throw new IllegalArgumentException("operator");
		if (number <= 0)
			throw new IllegalArgumentException("number");
		if (days == null || days.length == 0)
			throw new IllegalArgumentException("days");
		if (stops == null || stops.size() == 0)
			throw new IllegalArgumentException("stops");

		this.operator = operator;
		this.number = number;
		this.days = days;
		this.stops = stops;
	}

	public Duration travelDuration(Station departureStation,
			Station arrivalStation) {
		if ((departureStation == null) || (arrivalStation == null)) {
			throw new IllegalArgumentException(
					"Null argument in departure or arrival station");
		}

		Optional<Stop> fermataPartenza = Stop.searchByStation(stops,
				departureStation);
		Optional<Stop> fermataArrivo = Stop.searchByStation(stops,
				arrivalStation);

		if (!fermataPartenza.isPresent() || !fermataArrivo.isPresent()) {
			throw new IllegalArgumentException(
					"Invalid departure or arrival station");
		}

		Optional<LocalTime> tStart = fermataPartenza.get().getDepartureTime();
		Optional<LocalTime> tEnd = fermataArrivo.get().getArrivalTime();

		if (!tStart.isPresent() || !tEnd.isPresent()
				|| tStart.get().compareTo(tEnd.get()) > 0) {
			throw new IllegalArgumentException(
					"Invalid departure or arrival time");
		}

		return Duration.between(tStart.get(), tEnd.get());
	}

	public Operator getOperator() {
		return operator;
	}

	public long getNumber() {
		return number;
	}

	public DayOfWeek[] getDays() {
		return days;
	}

	public List<Stop> getStops() {
		return stops;
	}

	public SortedSet<String> getServedCities() {
		SortedSet<String> cities = new TreeSet<>();
		for (Stop s : stops) {
			cities.add(s.getStation().getCityName());
		}
		return cities;
	}

	public List<Station> getServedStations() {
		List<Station> stations = new ArrayList<>();
		for (Stop s : stops) {
			stations.add(s.getStation());
		}
		return stations;
	}

	@Override
	public String toString() {
		// ZannoRail 9902 Bologna C.le (6.18) / Torino P.S. (8.33)
		return operator.getFullName() + " " + number + " "
				+ stops.get(0).getStation().toString() + " ("
				+ stops.get(0).getDepartureTime().get().toString() + ")"
				+ " / " + stops.get(stops.size() - 1).getStation().toString()
				+ " ("
				+ stops.get(stops.size() - 1).getArrivalTime().get().toString()
				+ ")";
	}

	public String toFullString() {
		// ZannoRail 9902 Bologna C.le (6.18) / Torino P.S. (8.33)
		StringBuilder sb = new StringBuilder();
		sb.append(operator.getFullName());
		sb.append(" ");
		sb.append(number);
		sb.append(" da ");
		sb.append(stops.get(0).getStation().toString());
		sb.append(" (");
		sb.append(stops.get(0).getDepartureTime().get().toString());
		sb.append(")");
		sb.append(" a ");
		sb.append(stops.get(stops.size() - 1).getStation().toString());
		sb.append(" (");
		sb.append(stops.get(stops.size() - 1).getArrivalTime().get().toString());
		sb.append(")");
		sb.append(", circola ");
		for (DayOfWeek d : days) {
			sb.append(dayFormatter.format(d));
			sb.append("-");
		}
		sb.deleteCharAt(sb.length() - 1);
		sb.append(System.lineSeparator());
		sb.append("ferma a: ");
		// sb.append(getServedStations());
		for (int i = 1; i < stops.size() - 1; i++) {
			sb.append(stops.get(i).getStation().toString());
			sb.append(" (");
			sb.append(stops.get(i).getArrivalTime().get().toString());
			sb.append("/");
			sb.append(stops.get(i).getDepartureTime().get().toString());
			sb.append(")");
			sb.append(i == stops.size() - 2 ? " " : ", ");
		}
		return sb.toString();
	}
}
